@@ -1,5 +1,6 @@
 import math
 import os
+import time
 
 import numpy as np
 import pandas as pd
@@ -465,19 +466,21 @@ def leave_one_out(data, random=False):
     Returns:
         DataFrame: DataFrame that have already by labeled by a col with "train", "test" or "valid".
     """
+    start_time = time.time()
     print("leave_one_out")
     data[DEFAULT_FLAG_COL] = "train"
     if random:
         data = sklearn.utils.shuffle(data)
     else:
-        data.sort_values(by=[DEFAULT_TIMESTAMP_COL], inplace=True)
+        data.sort_values(by=[DEFAULT_TIMESTAMP_COL], ascending=False, inplace=True)
 
-    users = data[DEFAULT_USER_COL].unique()
-    for u in tqdm(users):
-        interactions = data[data[DEFAULT_USER_COL] == u].index.values
-        data.loc[interactions[-1], DEFAULT_FLAG_COL] = "test"
-        data.loc[interactions[-2], DEFAULT_FLAG_COL] = "validate"
+    data.loc[
+        data.groupby([DEFAULT_USER_COL]).head(2).index, DEFAULT_FLAG_COL
+    ] = "validate"
+    data.loc[data.groupby([DEFAULT_USER_COL]).head(1).index, DEFAULT_FLAG_COL] = "test"
 
+    end_time = time.time()
+    print(f"leave_one_out time cost: {end_time - start_time}")
     return data
 
 
@@ -516,7 +519,7 @@ def temporal_split(data, test_rate=0.1, by_user=False):
         test_rate (float): percentage of the test data.
             Note that percentage of the validation data will be the same as testing.
         by_user (bool): bool. Default False.
-                    - Ture: user-based split,
+                    - True: user-based split,
                     - False: global split,
 
     Returns:
@@ -566,7 +569,7 @@ def temporal_basket_split(data, test_rate=0.1, by_user=False):
         test_rate (float): percentage of the test data.
             Note that percentage of the validation data will be the same as testing.
         by_user (bool): Default False.
-                    - Ture: user-based split,
+                    - True: user-based split,
                     - False: global split,
 
     Returns:
